package koeln.uni.idh.java1.session11.zoo.animals;

import koeln.uni.idh.java1.session11.zoo.IsDrawable;
import koeln.uni.idh.java1.session11.zoo.Util;
import koeln.uni.idh.java1.session11.zoo.Zoo;

public abstract class Animal implements IsDrawable {
	static char[] directions = new char[] {'l', 't', 'r', 'b'};
	
	String name;

	int[] position;
	
	Zoo zoo;

	public abstract Animal reproduce(Animal partner);

	public abstract int getStepSize();
	
	public abstract char getSymbol();

	public abstract boolean canReproduce(Animal other);
	

	int direction;

	public Animal(Zoo zoo, String name, int[] initialPosition) {
		this.direction = Util.randInt(0, directions.length);
		this.zoo = zoo;
		this.name = name;
		this.position = initialPosition;
		System.out.println(this.getClass().getSimpleName() + " " + name + " is born at (" +this.position[0]+","+this.position[1]+ "). It's looking towards " + directions[direction] + ".");

	}

	public int getX() {
		return position[0];
	}

	public int getY() {
		return position[1];
	}

	
	public void tick() {
		int[] newPos = this.position.clone();
		
		switch (direction) {
		case 0:
			newPos[0] = newPos[0] - getStepSize();
			break;
		case 1:
			newPos[1] = newPos[1] - getStepSize();
			break;
		case 2:
			newPos[0] = newPos[0] + getStepSize();
			break;
		case 3:
			newPos[1] = newPos[1] + getStepSize();
			break;
		}
		
		if (zoo.isFree(newPos)) {
			this.position = newPos;
			System.out.println(this.getClass().getSimpleName() + " " + name + " has moved.");
		} else {
			Animal other = zoo.getAnimalByPosition(newPos);
			
			if (other != null && other != this && canReproduce(other)) {
				Animal child = this.reproduce(other);
				zoo.addAnimal(child);
				System.out.println("The two " + this.getClass().getSimpleName()+"s " + this.name + " and " + other.name + " met, fell in love and produced " + child.name+" as offspring.");
			} else {
				this.turn(Util.randInt(-1, 1));
				System.out.println(this.getClass().getSimpleName() + " " + name + " has turned and is now facing " + directions[direction]);
			}
		}

	}
	
	public void turn(int d) {
		direction = (direction + 4 + (int)Math.signum(d))%4;
	}
	
	protected int[] getFreePositionThatIsClose() {
		for (int xd = -1; xd < 2; xd++) {
			for (int yd = -1; yd < 2; yd++) {
				if (xd == 0 && yd == 0)
					continue;
				int[] ppos = this.position.clone();
				ppos[0] = ppos[0] + xd;
				ppos[1] = ppos[1] + yd;
				if (zoo.isFree(ppos)) {
					return ppos;
				}
			}
		}
		return zoo.getRandomFreePosition();
	}

}
